﻿using System;
using static XFUDriver.XFUDriver;

namespace DA_AD_QEI_LIO_CSharp
{
    partial class Program
    {
        static byte[] g_u8RxPDO = new byte[1400];
        static byte[] g_u8TxPDO = new byte[1400];
        static void Main(string[] args)
        {
            #region Init
            int nRet = 0;
            byte u8Version = 0;
            ushort u16SpiSize = 992;

            XF_SetDevType(0);   // 第一個參數為 0 for USB, 1 for PCIe

            nRet = UserSys_Init(24000000);
            if (nRet < 0)
            {
                Console.WriteLine("UserSys_Init return {0} Any key to exit.", nRet);
                Console.ReadKey();
                return;
            }
            u8Version = ECM_InitLibrary(ref u16SpiSize, ECM_CRC_TYPE.NONE);
            Console.WriteLine("u8Version 0x{0:X}", u8Version); //IC內部韌體版本
            if (u8Version == 0)
                return;
            #endregion

            #region LIO - Output
            Console.WriteLine("Set all GPIO pins on the chip as the output pin. Any key to set all outputs to HIGH.");
            Console.ReadKey();
            for (byte i = 0; i < 16; i++)
            {
                nRet = ECM_GpioSetMode(i, ECM_GPIO_MODE_OUTPUT, ECM_GPIO_PUSEL_DISABLE);
                if (nRet < 0)
                {
                    Console.WriteLine("GpioSetMode {0} Failed", i);
                    Console.ReadKey();
                }
            }
            nRet = ECM_GpioSetValue(0xffff);//GPIO high
            if (nRet < 0)
            {
                Console.WriteLine("GpioSetValue Failed");
                Console.ReadKey();
            }
            Console.WriteLine("Any key to set all outputs to LOW.");
            Console.ReadKey();
            nRet = ECM_GpioSetValue(0x0);//GPIO low
            if (nRet < 0)
            {
                Console.WriteLine("GpioSetValue Failed");
                Console.ReadKey();
            }
            Console.WriteLine("The output test finish. Any key to continue the input test.");
            Console.ReadKey();
            #endregion

            #region LIO - Input / interrupt
            UInt32 u32CompInt = 0;
            byte u8GPIOInt = 0, u8PeripInt = 0, u8ActiveHigh = 0;
            ushort inputValue = 0, intFlag = 0;
            Console.WriteLine("Set all GPIO pins on the chip as the input pin. ");
            for (byte i = 0; i < 16; i++)
            {
                nRet = ECM_GpioSetMode(i, ECM_GPIO_MODE_INPUT, ECM_GPIO_PUSEL_PULL_DOWN);
                if (nRet < 0)
                {
                    Console.WriteLine("GpioSetMode {0} Failed\r\n", i);
                    Console.ReadKey();
                }
                nRet = ECM_GpioIntEnable(i, ECM_GPIO_INT_FALLING);
                if (nRet < 0)
                {
                    Console.WriteLine("GpioIntEnable {0} Failed\r\n", i);
                    Console.ReadKey();
                }
            }
            nRet = ECM_SetEcatIntEnable(0, 0xFF, 0, 0); //set GPIO0~7 as interrupt source
            if (nRet <= 0)
            {
                Console.WriteLine("ECM_SetEcatIntEnable Failed");
                Console.ReadKey();
            }
            nRet = ECM_GetEcatIntEnable(ref u32CompInt, ref u8GPIOInt, ref u8PeripInt, ref u8ActiveHigh);
            if (nRet < 0)
            {
                Console.WriteLine("ECM_GetEcatIntEnable failed");
                Console.ReadKey();
            }
            Console.WriteLine("Int1 u32CompInt = 0x{0:X}, u8GPIOInt = 0x{1:X}, u8PeripInt = 0x{2:X}, u8ActiveHigh=0x{3:X}", u32CompInt, u8GPIOInt, u8PeripInt, u8ActiveHigh);
            for (int countDown = 50000; countDown >= 0; countDown--)
            {
                nRet = ECM_GpioGetValue(ref inputValue);
                if (nRet < 0)
                {
                    Console.WriteLine("ECM_GpioGetValue failed");
                    Console.ReadKey();
                }
                nRet = ECM_GpioGetIntFlag(ref intFlag);
                Console.Write("Input value is 0x{0:X}, intFlag = 0x{1:X}    After {2} counts finish.           \r", inputValue, intFlag, countDown);
                if (nRet < 0)
                {
                    Console.WriteLine("GpioGetIntFlag failed");
                    Console.ReadKey();
                }
                if (countDown % 10000 == 0 & intFlag != 0)
                {
                    Console.WriteLine("\nClear ch");
                    for (byte i = 0; i < 16; i++)
                    {
                        if ((intFlag & (ushort)Math.Pow(2, i)) != 0)
                        {
                            nRet = ECM_GpioIntClear(i);
                            Console.Write(i + ", ");
                        }
                        if (nRet < 0)
                        {
                            Console.WriteLine("ECM_GpioIntClear failed");
                            Console.ReadKey();
                        }
                    }
                    Console.WriteLine("");
                }
            }
            #endregion

            #region DAC / ADC
            //chip DAC & ADC test
            Console.WriteLine("Connect DAC pin to ADC pin. Any key to start ADC / DAC test.");
            Console.ReadKey();
            nRet = ECM_DacOpen(ECM_DAC_TRIG_MODE_SOFTWARE);
            if (nRet < 0)
            {
                Console.WriteLine("DacOpen failed");
                Console.ReadKey();
            }
            nRet = ECM_AdcOpen(ECM_ADC_MODE_SINGLE_END);
            if (nRet < 0)
            {
                Console.WriteLine("AdcOpen failed");
                Console.ReadKey();
            }

            for (uint i = 1; i < 257; i += 5)
            {
                uint voltageScale = 0;
                UInt32 data = 0, valid = 0;

                voltageScale = 16 * i - 1;

                nRet = ECM_DacSetData(voltageScale);
                if (nRet < 0)
                {
                    Console.WriteLine("DacSetData failed");
                    Console.ReadKey();
                }
                nRet = ECM_DacSetDelayTime(1023);
                if (nRet < 0)
                {
                    Console.WriteLine("DacSetDelayTime failed");
                    Console.ReadKey();
                }
                nRet = ECM_DacStartConv();
                if (nRet < 0)
                {
                    Console.WriteLine("DacStartConv failed");
                    Console.ReadKey();
                }
                nRet = ECM_AdcConfigSampleModule(ECM_ADC_TRIG_SOFTWARE);
                if (nRet < 0)
                {
                    Console.WriteLine("AdcConfigSampleModule failed");
                    Console.ReadKey();
                }
                nRet = ECM_AdcStartConv();
                if (nRet < 0)
                {
                    Console.WriteLine("AdcStartConv failed");
                    Console.ReadKey();
                }

                do
                {
                    nRet = ECM_AdcGetDataValidFlag(ref valid);
                    if (nRet < 0)
                    {
                        Console.WriteLine("AdcGatDataValidFlag failed");
                        Console.ReadKey();
                    }
                } while (valid == 0);

                nRet = ECM_AdcGetConvData(ref data);
                if (nRet < 0)
                {
                    Console.WriteLine("AdcGetConData failed");
                    Console.ReadKey();
                }
                Console.Write("DAC = {0}, ADC = {1}, reflash every 0.5 s    \r", voltageScale, data);//the value should be around the value is set as DacSetData
                System.Threading.Thread.Sleep(500);
            }
            #endregion

            #region QEI (ENC)
            //chip QEI (ENC) test
            Console.WriteLine("\nAny key to start QEI test.");
            Console.ReadKey();
            nRet = ECM_EncOpen(ECM_ENC_MODE_X4_FREE);
            if (nRet < 0)
            {
                Console.WriteLine("EncOpen failed");
                Console.ReadKey();
            }
            else
            {
                Console.WriteLine("EncOpen success");
            }
            nRet = ECM_EncStart();
            if (nRet < 0)
            {
                Console.WriteLine("EncStart failed");
                Console.ReadKey();
            }
            else
            {
                Console.WriteLine("EncStart success");
            }
            UInt32 count = 0;
            for (int countDown = 120; countDown >= 0; countDown--)
            {

                nRet = ECM_EncGetCount(ref count);
                if (nRet < 0)
                {
                    Console.WriteLine("EncGetCount failed");
                    Console.ReadKey();
                }
                else
                {
                    Console.Write("Count value is {0}, reflash every 0.5 s. After {1} counts finish.          \r", count, countDown);
                    System.Threading.Thread.Sleep(500);
                }
            }
            #endregion

            #region Clsoe the system
            Console.WriteLine("\nAny key to close.");
            Console.ReadKey();
            UserSys_Close();
            #endregion
        }
    }
}