﻿using System;
using static XFUDriver.XFUDriver;

namespace Drive_CSP_CSharp
{
    partial class Program
    {
        static byte[] g_u8RxPDO = new byte[1400];
        static byte[] g_u8TxPDO = new byte[1400];
        static void Main(string[] args)
        {
            #region PDO mapping
            PDO_CONFIG_HEAD[] rxPDOs = new PDO_CONFIG_HEAD[2];
            rxPDOs[0].slave = 0;
            rxPDOs[0].SmaIdx = 0x1C12;
            rxPDOs[0].PDOCnt = 1;
            rxPDOs[0].MapIdx = new UInt16[3];
            rxPDOs[0].MapIdx[0] = 0x1600;
            rxPDOs[0].ObjsCnt = new UInt16[3];
            rxPDOs[0].ObjsCnt[0] = 2;
            rxPDOs[0].Table = new OBJ_ENTRY_T[3, 8];
            rxPDOs[0].Table[0, 0].u8BitSize = 16;
            rxPDOs[0].Table[0, 0].u8SubIdx = 0;
            rxPDOs[0].Table[0, 0].u16Idx = 0x6040;  //Control Word
            rxPDOs[0].Table[0, 1].u8BitSize = 32;
            rxPDOs[0].Table[0, 1].u8SubIdx = 0;
            rxPDOs[0].Table[0, 1].u16Idx = 0x607A;  //Target Position

            PDO_CONFIG_HEAD[] txPDOs = new PDO_CONFIG_HEAD[2];
            txPDOs[0].slave = 0;
            txPDOs[0].SmaIdx = 0x1C13;
            txPDOs[0].PDOCnt = 1;
            txPDOs[0].MapIdx = new UInt16[3];
            txPDOs[0].MapIdx[0] = 0x1A00;
            txPDOs[0].ObjsCnt = new UInt16[3];
            txPDOs[0].ObjsCnt[0] = 2;
            txPDOs[0].Table = new OBJ_ENTRY_T[3, 8];
            txPDOs[0].Table[0, 0].u8BitSize = 16;
            txPDOs[0].Table[0, 0].u8SubIdx = 0;
            txPDOs[0].Table[0, 0].u16Idx = 0x6041;  // Status Word
            txPDOs[0].Table[0, 1].u8BitSize = 32;
            txPDOs[0].Table[0, 1].u8SubIdx = 0;
            txPDOs[0].Table[0, 1].u16Idx = 0x6064;  // Position actual value
            # endregion
            #region Init
            int nRet = 0;
            byte u8Version = 0, u8TmpEcState = 0, u8CrcErrCnt = 0, u8FifoCnt = 0, u8WkcErrCnt = 0, u8IsSlvAlive = 0;
            byte u8TxFIFOCnt = 64, u8RxFIFOCnt = 64;
            ushort u16RxPDOSize, u16TxPDOSize;
            uint u32InfoBuf = 0;
            sbyte nSlvCntOnline;
            int bLinkPin = -1;
            ushort u16SpiSize = 992;

            XF_SetDevType(0);   // 第一個參數為 0 for USB, 1 for PCIe

            nRet = UserSys_Init(24000000);
            if (nRet < 0)
            {
                Console.WriteLine("UserSys_Init return {0} Any key to exit.", nRet);
                Console.ReadKey();
                return;
            }
            u8Version = ECM_InitLibrary(ref u16SpiSize, ECM_CRC_TYPE.NONE);
            Console.WriteLine("u8Version 0x{0:X}", u8Version); //IC內部韌體版本
            if (u8Version == 0)
                return;
            nRet = ECM_GetNetLinkPin(ref bLinkPin);
            Console.WriteLine("ECM_GetNetLinkPin linkpin = {0}, ret={1}", bLinkPin, nRet); //linkpin=0 代表網路接口沒有連結

            nRet = ECM_EcatInit(0x300, 1000000); // 第一個參數0x300代表DC Sync, 0代表Free Run (依從站ESI說明設置), 第二個參數為週期時間(單位ns)
            Console.WriteLine("ECM_EcatInit return {0}", nRet); //return 1 代表 Init成功
            #endregion
            #region get slave info and set fifo and PDO maps
            nSlvCntOnline = ECM_EcatSlvCntGet();
            Console.WriteLine("Found {0} slave(s)", nSlvCntOnline);
            for (byte i = 0; i < nSlvCntOnline; i++)
            {
                nRet = ECM_SlaveInfoGet(i, ECM_SLV_INFO.ALstatuscode, ref u32InfoBuf);
                if (nRet > 0) //nRet: Slave Information Size
                {
                    if (u32InfoBuf != 0)
                    {
                        Console.WriteLine("Slave:{0} AL StatusCode:{1}...", i, u32InfoBuf); //AL StatusCode 非0 代表該Slave存在錯誤
                    }
                }
                else
                {
                    Console.WriteLine("Slave {0} Get AL StatusCode failed...", i);
                }
            }

            for (byte i = 0; i < 2; i++) // example for 2 driver slaves.
            {
                rxPDOs[0].slave = i;
                txPDOs[0].slave = i;
                SetPDOInfo(rxPDOs[0]);
                SetPDOInfo(txPDOs[0]);
            }

            nRet = ECM_EcatReconfig();
            if (nRet < 0)
            {
                Console.WriteLine("ECM_ECAT_CONFIG error : " + nRet + "\n");
                return;
            }

            ECM_SetTxFIFOCnt(u8TxFIFOCnt); //Slave -> Master
            ECM_SetRxFIFOCnt(u8RxFIFOCnt); //Master -> Slave
            ECM_InitFIFO();
            // RxPDOSize MUST be the same as RxPDO struct size
            u16RxPDOSize = ECM_FifoRxPdoSizeGet();
            Console.Write("RxPDOSize {0} Byte(s), RxPDO struct size {1}\n", u16RxPDOSize, System.Runtime.InteropServices.Marshal.SizeOf(cmdData));
            // TxPDOSize MUST be the same as TxPDO struct size
            u16TxPDOSize = ECM_FifoTxPdoSizeGet();
            Console.Write("TxPDOSize {0} Byte(s), TxPDO struct size {1}\n", u16TxPDOSize, System.Runtime.InteropServices.Marshal.SizeOf(respData));

            for (byte i = 0; i < 2; i++) // example for 2 driver slaves. (each slave for one motor.)
            {
                ECM_Drv402SM_AdvConfig(i, i, 0, 0);
                ECM_Drv402SM_StateSet(i, CIA402_SW.SWITCHEDONDISABLED);
            }
            nRet = ECM_EcatStateUpdate();
            if (nRet <= 0)
            {
                Console.Write("ECM_EcatStateUpdate fail({0})\n", nRet);
            }
            else
            {
                Console.Write("ECM_EcatStateUpdate done\n");
            }
            for (byte i = 0; i < nSlvCntOnline; i++)
            {
                nRet = ECM_EcatStateGet(i, ref u8TmpEcState);
                if (nRet > 0)
                {
                    Console.Write("slave {0} EcState:0x{1:X}\n", i, u8TmpEcState); // u8TmpEcState: 1 Init, 2 preOP, 4 SafeOP, 8 OP
                }
            }

            Console.Write("Any key to continue\n");
            Console.ReadKey();
            #endregion
            #region control state from perOP to OP
            ECM_EcatStateSet(0xFF, EC_STATE_SAFE_OP);  //0xFF 代表所有slave
            System.Threading.Thread.Sleep(1000);
            nRet = ECM_EcatStateGet(0xFF, ref u8TmpEcState); //0xFF 代表所有slave
            if (nRet > 0)
            {
                Console.Write("EcState:0x{0:X}\n", u8TmpEcState);
            }
            ECM_EcatStateSet(0xFF, EC_STATE_OPERATIONAL);
            for (byte i = 0; i < 10; i++)
            {
                System.Threading.Thread.Sleep(1000);
                nRet = ECM_EcatStateGet(0xFF, ref u8TmpEcState);
                if (nRet > 0)
                {
                    Console.Write("EcState:0x{0:X}\n", u8TmpEcState);
                }
                if (u8TmpEcState == 8)  // u8TmpEcState: 1 Init, 2 preOP, 4 SafeOP, 8 OP
                {
                    break;
                }
            }
            #endregion
            #region SDO write and read
            byte opMode = 8; // OP Mode:8 CSP
            for (byte i = 0; i < 2; i++) // example for 2 driver slaves. (each slave for one motor.)
            {
                nRet = ECM_EcatSdoWrite(i, 0x6060, 0, sizeof(byte), 10000000, ref opMode);
                nRet = ECM_EcatSdoRead(i, 0x6061, 0, sizeof(byte), 10000000, ref opMode);
                Console.WriteLine(i + ": opMode : " + opMode);
            }
            #endregion

            #region Get current and set target position before servo on
            ECM_ClearFIFO(0); // 參數0代表清除RxFIFO及TxFIFO, 1代表僅清除TxFIFO, 2代表僅清除RxFIFO
            for (int i = 0; i < 10; i++)
            {
                nRet = ECM_EcatPdoFifoDataExchangeAdv(ECM_FIFO_RW, 1, g_u8RxPDO, g_u8TxPDO, u16RxPDOSize, ref u8FifoCnt, ref u8CrcErrCnt, ref u8WkcErrCnt, ref u8IsSlvAlive);
                fromBytes(); // g_u8TxPDO -> respData           
                cmdData.driver0.Target1 = respData.driver0.Current1;
                cmdData.driver1.Target1 = respData.driver1.Current1;
                toBytes();
            }
            Console.Write("0: current Position {0}  \r\n", respData.driver0.Current1);  //respData.Current1 : Current Position
            Console.Write("1: current Position {0}  \r\n", respData.driver1.Current1);  //respData.Current1 : Current Position

            #endregion

            #region servo on
            for (byte i = 0; i < 2; i++) // example for 2 driver slaves.
            {
                ECM_Drv402SM_StateSet(i, CIA402_SW.OPERATIONENABLED);
            }
            Console.WriteLine("Servo on. Any key to start.");
            Console.ReadKey();
            System.Threading.Thread.Sleep(100);
            #endregion
            #region Cyclic PDO change using FIFO
            int speed = 0;
            if (u8TmpEcState == 8) // check EcState is OP 
            {
                u8CrcErrCnt = 0;
                for (int i = 0; i < 10000;)
                {
                    if (i < 100)
                    {
                        speed = i;                        
                    }
                    else if (i > 10000 - 100)
                    {
                        speed = 10000 - i;                       
                    }

                    if (u8FifoCnt < u8RxFIFOCnt - 3)
                    {
                        lock (_threadLock)
                        {
                            nRet = ECM_EcatPdoFifoDataExchangeAdv(ECM_FIFO_RW, 1, g_u8RxPDO, g_u8TxPDO, u16RxPDOSize, ref u8FifoCnt, ref u8CrcErrCnt, ref u8WkcErrCnt, ref u8IsSlvAlive);
                        }                        
                        //nRet:0 上一筆非FIFO命令, 4 讀寫失敗, 5 讀成功, 6 寫成功,7 讀寫成功
                        if (nRet <= 4)
                        {
                            Console.Write("time {0}\t FifoCnt {1} CrcErrCnt {2} Ret {3}\r\n", i, u8FifoCnt, u8CrcErrCnt, nRet);
                        }
                        else
                        {
                            cmdData.driver0.Target1 += speed; // Set Target Position for slave0
                            cmdData.driver1.Target1 += speed * 2; // Set Target Position for slave1
                            toBytes(); // cmdData -> g_u8RxPDO
                            i++;
                        }
                    }
                    else
                    {
                        lock (_threadLock)
                        {
                            nRet = ECM_EcatPdoFifoDataExchangeAdv(ECM_FIFO_RD, 1, g_u8RxPDO, g_u8TxPDO, u16RxPDOSize, ref u8FifoCnt, ref u8CrcErrCnt, ref u8WkcErrCnt, ref u8IsSlvAlive);
                        }
                        fromBytes(); // g_u8TxPDO -> respData
                        Console.Write("time {0}\tFifoCnt {1} CrcErrCnt {2} Ret {3} current0 {4} current1{5}          \r", i, u8FifoCnt, u8CrcErrCnt, nRet, respData.driver0.Current1, respData.driver1.Current1);  //respData.Current1 : Current Position
                        System.Threading.Thread.Sleep(10);
                    }
                }
                Console.Write("\n Finish\t\tFifoCnt {0} CrcErrCnt {1} Ret {2}\r\n", u8FifoCnt, u8CrcErrCnt, nRet);
            }
            Console.Write("Any key to close.\n");
            Console.ReadKey();
            #endregion
            #region servo off
            lock (_threadLock)
            {
                for (byte i = 0; i < 2; i++) // example for 2 driver slaves.
                {
                    ECM_Drv402SM_StateSet(i, CIA402_SW.READYTOSWITCHON);
                }
            }
            Console.WriteLine("Servo off.");            
            System.Threading.Thread.Sleep(100);
            #endregion
            #region control state from OP to Init and clsoe the system
            ECM_EcatStateSet(0xFF, EC_STATE_SAFE_OP);
            System.Threading.Thread.Sleep(100);
            ECM_EcatStateSet(0xFF, EC_STATE_PRE_OP);
            System.Threading.Thread.Sleep(100);
            ECM_EcatStateSet(0xFF, EC_STATE_INIT);
            System.Threading.Thread.Sleep(100);
            ECM_CloseLibrary();
            UserSys_Close();
            #endregion
        }
        private static int SetPDOInfo(PDO_CONFIG_HEAD pdoInfo)
        {
            int nret;
            UInt32 u32Tmp;
            if (pdoInfo.SmaIdx != 0x1C12 && pdoInfo.SmaIdx != 0x1C13)
                return -1;
            // clear pdo count
            byte zero = 0;
            nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.SmaIdx, 0, sizeof(byte), 10000000, ref zero);
            if (nret <= 0)
            {
                return -2;
            }
            for (byte nPdoIdx = 0; nPdoIdx < pdoInfo.PDOCnt; nPdoIdx++)
            {
                if (pdoInfo.ObjsCnt[nPdoIdx] > 0)
                {
                    nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.MapIdx[nPdoIdx], 0, sizeof(byte), 10000000, ref zero);
                    for (byte nObjIdx = 0; nObjIdx < pdoInfo.ObjsCnt[nPdoIdx]; nObjIdx++)
                    {
                        u32Tmp = (UInt32)((pdoInfo.Table[nPdoIdx, nObjIdx].u16Idx * 0x10000) + (pdoInfo.Table[nPdoIdx, nObjIdx].u8SubIdx * 0x100) + (pdoInfo.Table[nPdoIdx, nObjIdx].u8BitSize));
                        nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.MapIdx[nPdoIdx], (byte)(1 + nObjIdx), sizeof(UInt32), 10000000, ref u32Tmp);
                        if (nret <= 0)
                        {
                            return -3;
                        }
                    }
                    nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.MapIdx[nPdoIdx], 0, sizeof(byte), 10000000, ref pdoInfo.ObjsCnt[nPdoIdx]);
                    if (nret <= 0)
                    {
                        return -4;
                    }
                }
                nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.SmaIdx, (byte)(1 + nPdoIdx), sizeof(UInt16), 10000000, ref pdoInfo.MapIdx[nPdoIdx]);
                if (nret <= 0)
                {
                    return -5;
                }

            }
            nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.SmaIdx, 0, sizeof(byte), 10000000, ref pdoInfo.PDOCnt);
            if (nret <= 0)
            {
                return -6;
            }
            return 0;
        }
    }
}
