﻿using System;
using static XFUDriver.XFUDriver;

namespace EEPROM_CSharp
{
    partial class Program
    {
        static byte[] g_u8RxPDO = new byte[1400];
        static byte[] g_u8TxPDO = new byte[1400];
        static void Main(string[] args)
        {
            #region Init
            int nRet = 0;
            byte u8Version = 0;
            int bLinkPin = -1;
            ushort u16SpiSize = 992;

            XF_SetDevType(0);   // 第一個參數為 0 for USB, 1 for PCIe

            nRet = UserSys_Init(24000000);
            if (nRet < 0)
            {
                Console.WriteLine("UserSys_Init return {0} Any key to exit.", nRet);
                Console.ReadKey();
                return;
            }
            u8Version = ECM_InitLibrary(ref u16SpiSize, ECM_CRC_TYPE.NONE);
            Console.WriteLine("u8Version 0x{0:X}", u8Version); //IC內部韌體版本
            if (u8Version == 0)
                return;
            nRet = ECM_GetNetLinkPin(ref bLinkPin);
            Console.WriteLine("ECM_GetNetLinkPin linkpin = {0}, ret={1}", bLinkPin, nRet); //linkpin=0 代表網路接口沒有連結

            nRet = ECM_EcatInit(0x0, 1000000); // 第一個參數0x300代表DC Sync, 0代表Free Run (依從站ESI說明設置), 第二個參數為週期時間(單位ns)
            Console.WriteLine("ECM_EcatInit return {0}", nRet); //return 1 代表 Init成功
            #endregion

            #region Read EEPROM
            const int EEP_SIZE = 512;
            byte[] data;
            byte[] EEPROM = new byte[EEP_SIZE];
            Console.WriteLine("uint8_t sii_data[] = {");

            for (int i = 0; i < EEP_SIZE / 4; i++)
            {
                /*
                 * Two step to get SII data.
                 *      ECM_EcatEepromReq(ECM_EEPROM_OP_RD, slave, address, 0, pu8EcmStatus);
                 *      ECM_EcatEepromGet( pu64Data, pu8EcmStatus);
                 * Note that the address must be 32-bit aligned.
                 * This function get 32-bit data per request.
                 */
                UInt64 u64Data = 0;
                ECM_EcatEepromReq(ECM_EEPROM_OP_RD,  0/* slave */, (UInt16)(i * 2) /* eeprom addr */,  0/* don't care if read op */, 700000000/*ns*/ );
                nRet = ECM_EcatEepromGet(ref u64Data);
                if (nRet <= 0)
                    return;
                if ((i % 4) == 0)
                    Console.WriteLine("");//PRINTF("\r\nADDR %04x:", 2*i);
                
                data = BitConverter.GetBytes(u64Data);
                Console.Write(" 0x{0:X2}, 0x{1:X2}, 0x{2:X2}, 0x{3:X2},", data[0], data[1], data[2], data[3]);
                EEPROM[i * 4] = data[0];
                EEPROM[i * 4 + 1] = data[1];
                EEPROM[i * 4 + 2] = data[2];
                EEPROM[i * 4 + 3] = data[3];
            }
            Console.WriteLine("\r\n};");
            #endregion

            #region Write EEPROM
            Console.WriteLine("Enter YES to write the EEPROM on slave. NOTE:The data will NOT be recovered!");

            if (Console.ReadLine() == "YES")
            {
                byte[] sii_data =
{
 0x05, 0x0e, 0x03, 0x44, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00,
 0x10, 0x10, 0x10, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x10, 0x80, 0x00, 0x80, 0x10, 0x80, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x00, 0x01, 0x00,
 0x0a, 0x00, 0x49, 0x00, 0x0c, 0x05, 0x4e, 0x45, 0x58, 0x54, 0x57, 0x0c, 0x4e, 0x45, 0x58, 0x54,
 0x57, 0x20, 0x50, 0x44, 0x49, 0x20, 0x49, 0x4f, 0x08, 0x53, 0x79, 0x6e, 0x63, 0x68, 0x72, 0x6f,
 0x6e, 0x02, 0x44, 0x43, 0x07, 0x54, 0x78, 0x50, 0x44, 0x4f, 0x20, 0x30, 0x13, 0x44, 0x69, 0x67,
 0x69, 0x74, 0x61, 0x20, 0x49, 0x6e, 0x70, 0x75, 0x74, 0x20, 0x42, 0x79, 0x74, 0x65, 0x20, 0x30,
 0x07, 0x54, 0x78, 0x50, 0x44, 0x4f, 0x20, 0x31, 0x13, 0x44, 0x69, 0x67, 0x69, 0x74, 0x61, 0x20,
 0x49, 0x6e, 0x70, 0x75, 0x74, 0x20, 0x42, 0x79, 0x74, 0x65, 0x20, 0x31, 0x07, 0x52, 0x78, 0x50,
 0x44, 0x4f, 0x20, 0x30, 0x14, 0x44, 0x69, 0x67, 0x69, 0x74, 0x61, 0x20, 0x4f, 0x75, 0x74, 0x70,
 0x75, 0x74, 0x20, 0x42, 0x79, 0x74, 0x65, 0x20, 0x30, 0x07, 0x52, 0x78, 0x50, 0x44, 0x4f, 0x20,
 0x31, 0x14, 0x44, 0x69, 0x67, 0x69, 0x74, 0x61, 0x20, 0x4f, 0x75, 0x74, 0x70, 0x75, 0x74, 0x20,
 0x42, 0x79, 0x74, 0x65, 0x20, 0x31, 0x1e, 0x00, 0x10, 0x00, 0x01, 0x00, 0x01, 0x02, 0x01, 0x27,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x01, 0x00, 0x11, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x28, 0x00, 0x02, 0x00, 0x01, 0x02,
 0x03, 0xff, 0x29, 0x00, 0x10, 0x00, 0x00, 0x10, 0x80, 0x00, 0x26, 0x00, 0x01, 0x01, 0x80, 0x10,
 0x80, 0x00, 0x22, 0x00, 0x01, 0x02, 0x00, 0x11, 0x00, 0x00, 0x64, 0x00, 0x01, 0x03, 0x00, 0x14,
 0x00, 0x00, 0x20, 0x00, 0x01, 0x04, 0x32, 0x00, 0x10, 0x00, 0x00, 0x1a, 0x01, 0x03, 0x00, 0x05,
 0x10, 0x00, 0x00, 0x60, 0x01, 0x06, 0x05, 0x08, 0x00, 0x00, 0x01, 0x1a, 0x01, 0x03, 0x00, 0x07,
 0x10, 0x00, 0x00, 0x60, 0x01, 0x08, 0x05, 0x08, 0x00, 0x00, 0x33, 0x00, 0x10, 0x00, 0x00, 0x16,
 0x01, 0x02, 0x00, 0x09, 0x10, 0x00, 0x00, 0x70, 0x01, 0x0a, 0x05, 0x08, 0x00, 0x00, 0x01, 0x16,
 0x01, 0x02, 0x00, 0x0b, 0x10, 0x00, 0x00, 0x70, 0x02, 0x0c, 0x05, 0x08, 0x00, 0x00, 0x3c, 0x00,
 0x18, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x01, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x01, 0x00, 0x04, 0x00, 0x00, 0x00,
 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
};
                for (ushort i = 0; i < sii_data.Length / 2; i++)
                {
                    UInt16 writeData =(UInt16)(sii_data[2 * i] + (sii_data[2 * i + 1] << 8));
                    ECM_EcatEepromReq(ECM_EEPROM_OP_WR, 0/* slave */, i/* eeprom addr */, writeData/* don't care if read op */, 700000000/*ns*/);
                }
                Console.WriteLine("Write EEPROM finish.");
            }
            #endregion

            #region Clsoe the system
            Console.WriteLine("\nAny key to close.");
            Console.ReadKey();
            UserSys_Close();
            #endregion
        }
    }
}