﻿using System;
using static XFUDriver.XFUDriver;

namespace IO_CSharp
{
    partial class Program
    {
        static string dtype2string(ec_datatype dtype, UInt16 bitlen)
        {
            string str;
            switch (dtype)
            {
                case ec_datatype.ECT_BOOLEAN:
                    str = "BOOLEAN";
                    break;
                case ec_datatype.ECT_INTEGER8:
                    str = "INTEGER8";
                    break;
                case ec_datatype.ECT_INTEGER16:
                    str = "INTEGER16";
                    break;
                case ec_datatype.ECT_INTEGER32:
                    str = "INTEGER32";
                    break;
                case ec_datatype.ECT_INTEGER24:
                    str = "INTEGER24";
                    break;
                case ec_datatype.ECT_INTEGER64:
                    str = "INTEGER64";
                    break;
                case ec_datatype.ECT_UNSIGNED8:
                    str =  "UNSIGNED8";
                    break;
                case ec_datatype.ECT_UNSIGNED16:
                    str = "UNSIGNED16";
                    break;
                case ec_datatype.ECT_UNSIGNED32:
                    str = "UNSIGNED32";
                    break;
                case ec_datatype.ECT_UNSIGNED24:
                    str = "UNSIGNED24";
                    break;
                case ec_datatype.ECT_UNSIGNED64:
                    str = "UNSIGNED64";
                    break;
                case ec_datatype.ECT_REAL32:
                    str = "REAL32";
                    break;
                case ec_datatype.ECT_REAL64:
                    str = "REAL64";
                    break;
                case ec_datatype.ECT_BIT1:
                    str = "BIT1";
                    break;
                case ec_datatype.ECT_BIT2:
                    str = "BIT2";
                    break;
                case ec_datatype.ECT_BIT3:
                    str = "BIT3";
                    break;
                case ec_datatype.ECT_BIT4:
                    str = "BIT4";
                    break;
                case ec_datatype.ECT_BIT5:
                    str = "BIT5";
                    break;
                case ec_datatype.ECT_BIT6:
                    str = "BIT6";
                    break;
                case ec_datatype.ECT_BIT7:
                    str = "BIT7";
                    break;
                case ec_datatype.ECT_BIT8:
                    str = "BIT8";
                    break;
                case ec_datatype.ECT_VISIBLE_STRING:
                    str = string.Format("VISIBLE_STR({0:d})", bitlen);
                    break;
                case ec_datatype.ECT_OCTET_STRING:
                    str = string.Format("OCTET_STR({0:d})", bitlen);
                    break;
                default:
                    str = string.Format("dt:0x{0,4.4:X} ({1:d})", dtype, bitlen);
                    break;
            }
            return str;
        }

        static string otype2string(UInt16 otype)
        {
            string str;
            switch (otype)
            {
                case OTYPE_VAR:
                    str = "VAR";
                    break;
                case OTYPE_ARRAY:
                    str = "ARRAY";
                    break;
                case OTYPE_RECORD:
                    str = "RECORD";
                    break;
                default:
                    str = string.Format("ot:0x{0,4.4:X}", otype);
                    break;
            }
            return str;
        }

        static string access2string(UInt16 access)
        {
            string str;
            str = string.Format("{0:s}{1:s}{2:s}{3:s}{4:s}{5:s}",
                ((access & ATYPE_Rpre) != 0 ? "R" : "_"),
                ((access & ATYPE_Wpre) != 0 ? "W" : "_"),
                ((access & ATYPE_Rsafe) != 0 ? "R" : "_"),
                ((access & ATYPE_Wsafe) != 0 ? "W" : "_"),
                ((access & ATYPE_Rop) != 0 ? "R" : "_"),
                ((access & ATYPE_Wop) != 0 ? "W" : "_"));
            return str;
        }
        static void getObjectInfo(byte slv)
        {
            UInt16 i;
            byte j;
            UInt16 entries=0;
            ECM_AllocODOEList();
            ECM_ReadODList(slv);
            ECM_GetODEntries(ref entries);

            Console.WriteLine("CoE Object Description found, {0:d} entries.", entries);
            for (i = 0; i < entries; i++)
            {
                UInt16 index=0;
                byte objcode=0;
                byte max_sub=0;
                System.Text.StringBuilder odname = new System.Text.StringBuilder(41);
                string name;               

                ECM_ReadODDesc(i);
                ECM_GetODIndex(ref index, i);
                ECM_GetODObjcode(ref objcode);
                ECM_GetODMaxSub(ref max_sub);
                ECM_GetODName(odname);
                name = string.Format("\"{0}\"", odname.ToString());
                if (objcode == OTYPE_VAR)
                {
                    Console.WriteLine("0x{0,04:x}      {1,-40:s}      [{2:s}]", index, name, otype2string(objcode));
                }
                else
                {
                    Console.WriteLine("0x{0,04:x}   " +
                        "    {1,-40:s}      [{2:s}  maxsub(0x{3,02:x})]", index, name, otype2string(objcode), max_sub);
                }

                for (j = 0; j < max_sub + 1; j++)
                {
                    UInt16 bitlength=0, objaccess=0, datatype=0;
                    ECM_ReadOE(i, j);
                    ECM_GetOEDataType(ref datatype);
                    ECM_GetOEBitLength(ref bitlength);
                    ECM_GetOEObjAccess(ref objaccess);
                    ECM_GetOEName(odname);
                    if ((datatype > 0) && (bitlength > 0))
                    {
                        name = string.Format("\"{0}\"", odname.ToString());
                        Console.WriteLine("    0x{0,02:x}      {1,-40:s}      [{2,-16:s} {3,6:s}]", j, name, dtype2string((ec_datatype)datatype, bitlength), access2string(objaccess));
                    }
                }
            }
            ECM_FreeODOEList();
        }
        static byte[] g_u8RxPDO = new byte[1400];
        static byte[] g_u8TxPDO = new byte[1400];
        static void Main(string[] args)
        {
            #region Init
            int nRet = 0;
            byte u8Version = 0, u8TmpEcState = 0, u8CrcErrCnt = 0, u8FifoCnt = 0, u8WkcErrCnt = 0, u8IsSlvAlive = 0;
            byte u8TxFIFOCnt = 64, u8RxFIFOCnt = 64;
            ushort u16RxPDOSize, u16TxPDOSize;
            uint u32InfoBuf = 0;
            sbyte nSlvCntOnline;
            int bLinkPin = -1;
            ushort u16SpiSize = 992;

            XF_SetDevType(0);   // 第一個參數為 0 for USB, 1 for PCIe

            nRet = UserSys_Init(24000000);
            if (nRet < 0)
            {
                Console.WriteLine("UserSys_Init return {0} Any key to exit.", nRet);
                Console.ReadKey();
                return;
            }
            u8Version = ECM_InitLibrary(ref u16SpiSize, ECM_CRC_TYPE.NONE);
            Console.WriteLine("u8Version 0x{0:X}", u8Version); //IC內部韌體版本
            if (u8Version == 0)
                return;
            nRet = ECM_GetNetLinkPin(ref bLinkPin);
            Console.WriteLine("ECM_GetNetLinkPin linkpin = {0}, ret={1}", bLinkPin, nRet); //linkpin=0 代表網路接口沒有連結

            nRet = ECM_EcatInit(0x0, 1000000); // 第一個參數0x300代表DC Sync, 0代表Free Run (依從站ESI說明設置), 第二個參數為週期時間(單位ns)
            Console.WriteLine("ECM_EcatInit return {0}", nRet); //return 1 代表 Init成功
            #endregion

            #region get slave info and set fifo
            nSlvCntOnline = ECM_EcatSlvCntGet();
            Console.WriteLine("Found {0} slave(s)", nSlvCntOnline);
            for (byte i = 0; i < nSlvCntOnline; i++)
            {
                nRet = ECM_SlaveInfoGet(i, ECM_SLV_INFO.ALstatuscode, ref u32InfoBuf);
                if (nRet > 0) //nRet: Slave Information Size
                {
                    if (u32InfoBuf != 0)
                    {
                        Console.WriteLine("Slave:{0} AL StatusCode:{1}...", i, u32InfoBuf); //AL StatusCode 非0 代表該Slave存在錯誤
                    }
                }
                else
                {
                    Console.WriteLine("Slave {0} Get AL StatusCode failed...", i);
                }
            }

            ECM_SetTxFIFOCnt(u8TxFIFOCnt); //Slave -> Master
            ECM_SetRxFIFOCnt(u8RxFIFOCnt); //Master -> Slave
            ECM_InitFIFO();
            // RxPDOSize MUST be the same as RxPDO struct size
            u16RxPDOSize = ECM_FifoRxPdoSizeGet();
            Console.Write("RxPDOSize {0} Byte(s), RxPDO struct size {1}\n", u16RxPDOSize, System.Runtime.InteropServices.Marshal.SizeOf(cmdData));
            // TxPDOSize MUST be the same as TxPDO struct size
            u16TxPDOSize = ECM_FifoTxPdoSizeGet();
            Console.Write("TxPDOSize {0} Byte(s), TxPDO struct size {1}\n", u16TxPDOSize, System.Runtime.InteropServices.Marshal.SizeOf(respData));

            nRet = ECM_EcatStateUpdate();
            if (nRet <= 0)
            {
                Console.Write("ECM_EcatStateUpdate fail({0})\n", nRet);
            }
            else
            {
                Console.Write("ECM_EcatStateUpdate done\n");
            }
            for (byte i = 0; i < nSlvCntOnline; i++)
            {
                nRet = ECM_EcatStateGet(i, ref u8TmpEcState);
                if (nRet > 0)
                {
                    Console.Write("slave {0} EcState:0x{1:X}\n", i, u8TmpEcState); // u8TmpEcState: 1 Init, 2 preOP, 4 SafeOP, 8 OP
                }
            }

            Console.Write("Any key to continue\n");
            Console.ReadKey();
            #endregion
            #region Get Slave PDO Information
            getObjectInfo(0);// get slave 0 PDO information
            Console.WriteLine("============================\r\n Any key to finish.");
            Console.ReadKey();
            #endregion

            #region control state to Init and clsoe the system
            ECM_EcatStateSet(0xFF, EC_STATE_INIT);
            System.Threading.Thread.Sleep(100);
            ECM_CloseLibrary();
            UserSys_Close();
            #endregion
        }
        private static int SetPDOInfo(PDO_CONFIG_HEAD pdoInfo)
        {
            int nret;
            UInt32 u32Tmp;
            if (pdoInfo.SmaIdx != 0x1C12 && pdoInfo.SmaIdx != 0x1C13)
                return -1;
            // clear pdo count
            byte zero = 0;
            nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.SmaIdx, 0, sizeof(byte), 10000000, ref zero);
            if (nret <= 0)
            {
                return -2;
            }
            for (byte nPdoIdx = 0; nPdoIdx < pdoInfo.PDOCnt; nPdoIdx++)
            {
                if (pdoInfo.ObjsCnt[nPdoIdx] > 0)
                {
                    nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.MapIdx[nPdoIdx], 0, sizeof(byte), 10000000, ref zero);
                    for (byte nObjIdx = 0; nObjIdx < pdoInfo.ObjsCnt[nPdoIdx]; nObjIdx++)
                    {
                        u32Tmp = (UInt32)((pdoInfo.Table[nPdoIdx, nObjIdx].u16Idx * 0x10000) + (pdoInfo.Table[nPdoIdx, nObjIdx].u8SubIdx * 0x100) + (pdoInfo.Table[nPdoIdx, nObjIdx].u8BitSize));
                        nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.MapIdx[nPdoIdx], (byte)(1 + nObjIdx), sizeof(UInt32), 10000000, ref u32Tmp);
                        if (nret <= 0)
                        {
                            return -3;
                        }
                    }
                    nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.MapIdx[nPdoIdx], 0, sizeof(byte), 10000000, ref pdoInfo.ObjsCnt[nPdoIdx]);
                    if (nret <= 0)
                    {
                        return -4;
                    }
                }
                nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.SmaIdx, (byte)(1 + nPdoIdx), sizeof(UInt16), 10000000, ref pdoInfo.MapIdx[nPdoIdx]);
                if (nret <= 0)
                {
                    return -5;
                }

            }
            nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.SmaIdx, 0, sizeof(byte), 10000000, ref pdoInfo.PDOCnt);
            if (nret <= 0)
            {
                return -6;
            }
            return 0;
        }
    }
}
