﻿/******************************************************************************
 *	File	:	Driver_CSP.cpp
 *	Version :	1.7
 *	Date	:	2022/07/27
 *	Author	:	User
 *
 *	Hardware dependency function - C source file
 *
 *	Provide necessary function for EcmUsrDriver
 *
 * @copyright (C) 2020 NEXTW TECHNOLOGY CO., LTD.. All rights reserved.
 *
 ******************************************************************************/
#include <windows.h>
#include <iostream>
#include <mutex>
#include <platform.h>
#include <EcmUsrDriver.h>
#include "PdoDefine.h"
uint8_t g_u8RxPDO[1400];
uint8_t g_u8TxPDO[1400];

int SetPDOInfo(PDO_CONFIG_HEAD pdoInfo)
{
	int nret;
	uint32_t u32Tmp;
	if (pdoInfo.SmaIdx != 0x1C12 && pdoInfo.SmaIdx != 0x1C13)
		return -1;
	// clear pdo count
	byte zero = 0;
	nret = ECM_EcatSdoWrite(pdoInfo.Slave, pdoInfo.SmaIdx, 0, sizeof(byte), 10000000, &zero);
	if (nret <= 0)
	{
		return -2;
	}
	for (byte nPdoIdx = 0; nPdoIdx < pdoInfo.PDOCnt; nPdoIdx++)
	{
		if (pdoInfo.ObjsCnt[nPdoIdx] > 0)
		{
			nret = ECM_EcatSdoWrite(pdoInfo.Slave, pdoInfo.MapIdx[nPdoIdx], 0, sizeof(byte), 10000000, &zero);
			for (byte nObjIdx = 0; nObjIdx < pdoInfo.ObjsCnt[nPdoIdx]; nObjIdx++)
			{
				u32Tmp = (pdoInfo.Table[nPdoIdx, nObjIdx].u16Idx * 0x10000) + (pdoInfo.Table[nPdoIdx, nObjIdx].u8SubIdx * 0x100) + (pdoInfo.Table[nPdoIdx, nObjIdx].u8BitSize);
				nret = ECM_EcatSdoWrite(pdoInfo.Slave, pdoInfo.MapIdx[nPdoIdx], (byte)(1 + nObjIdx), sizeof(uint32_t), 10000000, (uint8_t*)&u32Tmp);
				if (nret <= 0)
				{
					return -3;
				}
			}
			nret = ECM_EcatSdoWrite(pdoInfo.Slave, pdoInfo.MapIdx[nPdoIdx], 0, sizeof(byte), 10000000, (uint8_t*)&pdoInfo.ObjsCnt[nPdoIdx]);
			if (nret <= 0)
			{
				return -4;
			}
		}
		nret = ECM_EcatSdoWrite(pdoInfo.Slave, pdoInfo.SmaIdx, (byte)(1 + nPdoIdx), sizeof(uint16_t), 10000000, (uint8_t*)&pdoInfo.MapIdx[nPdoIdx]);
		if (nret <= 0)
		{
			return -5;
		}

	}
	nret = ECM_EcatSdoWrite(pdoInfo.Slave, pdoInfo.SmaIdx, 0, sizeof(byte), 10000000, (uint8_t*)&pdoInfo.PDOCnt);
	if (nret <= 0)
	{
		return -6;
	}
	return 0;
}

int main()
{
	#pragma region PDO mapping
	PDO_CONFIG_HEAD rxPDOs[2];
	rxPDOs[0].Slave = 0;
	rxPDOs[0].SmaIdx = 0x1C12;
	rxPDOs[0].PDOCnt = 1;
	rxPDOs[0].MapIdx[0] = 0x1600;
	rxPDOs[0].ObjsCnt[0] = 2;
	rxPDOs[0].Table[0, 0].u8BitSize = 16;
	rxPDOs[0].Table[0, 0].u8SubIdx = 0;
	rxPDOs[0].Table[0, 0].u16Idx = 0x6040;  //Control Word
	rxPDOs[0].Table[0, 1].u8BitSize = 32;
	rxPDOs[0].Table[0, 1].u8SubIdx = 0;
	rxPDOs[0].Table[0, 1].u16Idx = 0x607A;  ////Target Position

	PDO_CONFIG_HEAD txPDOs[2];
	txPDOs[0].Slave = 0;
	txPDOs[0].SmaIdx = 0x1C13;
	txPDOs[0].PDOCnt = 1;
	txPDOs[0].MapIdx[0] = 0x1A00;
	txPDOs[0].ObjsCnt[0] = 2;
	txPDOs[0].Table[0, 0].u8BitSize = 16;
	txPDOs[0].Table[0, 0].u8SubIdx = 0;
	txPDOs[0].Table[0, 0].u16Idx = 0x6041;  // Status Word
	txPDOs[0].Table[0, 1].u8BitSize = 32;
	txPDOs[0].Table[0, 1].u8SubIdx = 0;
	txPDOs[0].Table[0, 1].u16Idx = 0x6064;  // Position actual value
	#pragma endregion
	#pragma region Init
	std::mutex _lock;
	int bLinkPin;
	unsigned short u16SpiSize = 992;
	uint8_t u8State = 0, u8Version = 0, u8TmpEcState = 0, u8CrcErrCnt = 0, u8LastCrcErrCnt = 0, u8FifoCnt = 0, u8WkcErrCnt = 0, u8IsSlvAlive = 0;
	uint8_t u8TxFIFOCnt = 64, u8RxFIFOCnt = 64;
	uint16_t u16InfoBuf, u16RxPDOSize, u16TxPDOSize;
	TOTAL_RXPDO_ST_DEF_T	*pRxPDOData;
	TOTAL_TXPDO_ST_DEF_T	*pTxPDOData;
	int i = 0, nRet = 0;
	int nSlvCntOnline;
	memset(g_u8RxPDO, 0, sizeof(g_u8RxPDO));
	memset(g_u8TxPDO, 0, sizeof(g_u8TxPDO));
	pTxPDOData = (TOTAL_TXPDO_ST_DEF_T *)g_u8TxPDO;
	pRxPDOData = (TOTAL_RXPDO_ST_DEF_T *)g_u8RxPDO;

	// 第一個參數為 0 for USB
	// 第一個參數為 1 for PCIe
	XF_SetDevType(0);
	// 初始化PCIe晶片
	//
	nRet = UserSys_Init(24000000);
	if (nRet < 0) {
		printf("UserSys_Init return %d\n", nRet);
		return -1;
	}
	u8Version = ECM_InitLibrary(&u16SpiSize, ECM_CRC_TYPE_NONE);
	printf("u8Version 0x%X\n", u8Version);
	if (u8Version == 0)
		return 0;
	nRet = ECM_GetNetLinkPin(&bLinkPin); 
	printf("ECM_GetNetLinkPin linkpin(%d) ret(%d)\n", bLinkPin, nRet); //linkpin=0 代表網路接口沒有連結

	nRet = ECM_EcatInit(0x300, 500000);// 第一個參數0x300代表DC Sync, 0代表Free Run (依從站ESI說明設置), 第二個參數為週期時間(單位ns)
	printf("ECM_EcatInit %d\n", nRet); //return 1 代表 Init成功
	#pragma endregion
	#pragma region get slave info and set fifo and PDO maps
	nSlvCntOnline = ECM_EcatSlvCntGet();
	printf("Found %d slave(s)\n", nSlvCntOnline);
	for (i = 0; i < nSlvCntOnline; i++) {
		nRet = ECM_SlaveInfoGet(i, ECM_SLV_INFO_ALstatuscode, (uint8_t *)&u16InfoBuf);
		if (nRet > 0) { //nRet: Slave Information Size
			if (u16InfoBuf != 0) {
				printf("Slave %d AL StatusCode(%d)...\n", i, u16InfoBuf);
			}
		}
		else {
			printf("Slave %d Get AL StatusCode failed...\n", i);
		}
	}

	SetPDOInfo(rxPDOs[0]);
	SetPDOInfo(txPDOs[0]);
	nRet = ECM_EcatReconfig();
	if (nRet < 0)
	{
		printf("ECM_ECAT_CONFIG error : %d \n", nRet);
		return -1;
	}

	ECM_SetTxFIFOCnt(u8TxFIFOCnt); //Slave -> Master
	ECM_SetRxFIFOCnt(u8RxFIFOCnt); //Master -> Slave
	ECM_InitFIFO();

	// RxPDOSize = RxPDO struct size * TEST_PDO_TO_FIFO_ONCE
	u16RxPDOSize = ECM_FifoRxPdoSizeGet();
	printf("RxPDOSize %d Byte(s), RxPDO struct size %zd Byte(s)\n", u16RxPDOSize, sizeof(Total_rxpdo_st_def_t));
	// TxPDOSize = TxPDO struct size * TEST_PDO_TO_FIFO_ONCE
	u16TxPDOSize = ECM_FifoTxPdoSizeGet();
	printf("TxPDOSize %dByte(s), TxPDO struct size %zd Byte(s)\n", u16TxPDOSize, sizeof(Total_txpdo_st_def_t));

	ECM_Drv402SM_AdvConfig(0, 0, 0, 0);
	ECM_Drv402SM_StateSet(0, CIA402_SW_SWITCHEDONDISABLED);

	nRet = ECM_EcatStateUpdate();
	if (nRet <= 0) {
		printf("ECM_EcatStateUpdate fail(%d)\n", nRet);
	}
	else {
		printf("ECM_EcatStateUpdate done\n");
	}
	for (i = 0; i < nSlvCntOnline; i++) {
		nRet = ECM_EcatStateGet(i, &u8TmpEcState);
		if (nRet) {
			printf("slave (%d) EcState(0x%X)\n", i, u8TmpEcState);
		}
	}
	printf("Wait key\n");
	getchar();
	#pragma endregion
	#pragma region control state from perOP to OP
	ECM_EcatStateSet(0xFF, EC_STATE_SAFE_OP);
	Sleep(1000);
	nRet = ECM_EcatStateGet(0xFF, &u8TmpEcState);
	if (nRet) {
		printf("EcState(0x%X)\n", u8TmpEcState);
	}
	ECM_EcatStateSet(0xFF, EC_STATE_OPERATIONAL);
	for (i = 0; i < 10; i++) {
		Sleep(1000);
		nRet = ECM_EcatStateGet(0xFF, &u8TmpEcState);
		if (nRet) {
			printf("EcState(0x%X)\n", u8TmpEcState);
		}
		if (u8TmpEcState == 8) { // u8TmpEcState: 1 Init, 2 preOP, 4 SafeOP, 8 OP
			break;
		}
	}
	#pragma endregion
	#pragma region SDO write and read
	byte opMode = 8; // OP Mode:8 CSP
	nRet = ECM_EcatSdoWrite(0, 0x6060, 0, sizeof(byte), 10000000, &opMode);
	nRet = ECM_EcatSdoRead(0, 0x6060, 0, sizeof(byte), 10000000, &opMode);
	printf("opMode : %d \n", opMode);
	#pragma endregion
	#pragma region Get current and set target position before servo on
	ECM_ClearFIFO(0); // 參數0代表清除RxFIFO及TxFIFO, 1代表僅清除TxFIFO, 2代表僅清除RxFIFO
	for (int i = 0; i < 10; i++)
	{
		_lock.lock();
		nRet = ECM_EcatPdoFifoDataExchangeAdv(ECM_FIFO_RW, 1, g_u8RxPDO, g_u8TxPDO, u16RxPDOSize, &u8FifoCnt, &u8CrcErrCnt, &u8WkcErrCnt, &u8IsSlvAlive);
		pRxPDOData->CycleCmd[0].DRIVE_GROUP_0[0].n32TarPos = pTxPDOData->CycleResp[0].DRIVE_GROUP_0[0].n32AcuPos;
		_lock.unlock();
	}
	printf("current Position %d  \r\n", pTxPDOData->CycleResp[0].DRIVE_GROUP_0[0].n32AcuPos);  //respData.Current1 : Current Position
	#pragma endregion
	#pragma region servo on
	_lock.lock();
	ECM_Drv402SM_StateSet(0, CIA402_SW_OPERATIONENABLED);
	_lock.unlock();
	printf("Servo on. Any key to start.\n");
	getchar();
	Sleep(100);
	#pragma endregion
	#pragma region Cyclic PDO change using FIFO
	int speed = 0, lastTarget = pRxPDOData->CycleCmd[0].DRIVE_GROUP_0[0].n32TarPos;
	if (u8TmpEcState == 8) {	// check EcState is OP 
		u8CrcErrCnt = 0;
		for (i = 0; i < 100000; ) {
			if (i < 100)
			{
				speed = i; // Set Target Velocity 
			}
			else if (i > 100000 - 100)
			{
				speed = 100000 - i;
			}

			if (u8FifoCnt < u8RxFIFOCnt - 3 * TEST_PDO_TO_FIFO_ONCE) {
				_lock.lock();
				nRet = ECM_EcatPdoFifoDataExchangeAdv(ECM_FIFO_RW, TEST_PDO_TO_FIFO_ONCE, g_u8RxPDO, g_u8TxPDO, u16RxPDOSize, &u8FifoCnt, &u8CrcErrCnt, NULL, NULL);
				_lock.unlock();
				//nRet:0 上一筆非FIFO命令, 4 讀寫失敗, 5 讀成功, 6 寫成功,7 讀寫成功
				if (nRet <= 4) {
					printf("time %d:\tFifoCnt %d, CrcErrCnt %d, Ret %d\r\n", i, u8FifoCnt, u8CrcErrCnt, nRet);
				}
				else {
					pRxPDOData->CycleCmd[0].DRIVE_GROUP_0[0].n32TarPos = lastTarget + speed;
					for (int j = 0; j < TEST_PDO_TO_FIFO_ONCE - 1; j++)
					{
						pRxPDOData->CycleCmd[j+1].DRIVE_GROUP_0[0].n32TarPos = pRxPDOData->CycleCmd[j].DRIVE_GROUP_0[0].n32TarPos + speed; // Set Target Velocity                               
					}
					lastTarget = pRxPDOData->CycleCmd[TEST_PDO_TO_FIFO_ONCE - 1].DRIVE_GROUP_0[0].n32TarPos;					
					i++;
				}
			}
			else {
				_lock.lock();
				nRet = ECM_EcatPdoFifoDataExchangeAdv(ECM_FIFO_RD, 1, g_u8RxPDO, g_u8TxPDO, u16RxPDOSize, &u8FifoCnt, &u8CrcErrCnt, NULL, NULL);
				_lock.unlock();
				printf("time %d:\tFifoCnt %d, CrcErrCnt %d, Ret %d, current %d      \r", i, u8FifoCnt, u8CrcErrCnt, nRet, pTxPDOData->CycleResp[0].DRIVE_GROUP_0[0].n32AcuPos);
				Sleep(10);
			}
		}
		printf("\n Finish \tFifoCnt %d, CrcErrCnt %d, Ret %d\n", u8FifoCnt, u8CrcErrCnt, nRet);
		printf("Any key to close.\n");
		getchar();
	}
	#pragma endregion
	#pragma region servo off
	_lock.lock();
	ECM_Drv402SM_StateSet(0, CIA402_SW_READYTOSWITCHON);
	_lock.unlock();
	printf("Servo off.\n");
	Sleep(100);
	#pragma endregion
	#pragma region control state from OP to Init and clsoe the system
	ECM_EcatStateSet(0xFF, EC_STATE_SAFE_OP);
	Sleep(100);
	ECM_EcatStateSet(0xFF, EC_STATE_PRE_OP);
	Sleep(100);
	ECM_EcatStateSet(0xFF, EC_STATE_INIT);
	Sleep(100);
	ECM_CloseLibrary();
	UserSys_Close();
	#pragma endregion
}


// 執行程式: Ctrl + F5 或 [偵錯] > [啟動但不偵錯] 功能表
// 偵錯程式: F5 或 [偵錯] > [啟動偵錯] 功能表

// 開始使用的提示: 
//   1. 使用 [方案總管] 視窗，新增/管理檔案
//   2. 使用 [Team Explorer] 視窗，連線到原始檔控制
//   3. 使用 [輸出] 視窗，參閱組建輸出與其他訊息
//   4. 使用 [錯誤清單] 視窗，檢視錯誤
//   5. 前往 [專案] > [新增項目]，建立新的程式碼檔案，或是前往 [專案] > [新增現有項目]，將現有程式碼檔案新增至專案
//   6. 之後要再次開啟此專案時，請前往 [檔案] > [開啟] > [專案]，然後選取 .sln 檔案
