﻿/******************************************************************************
 *	File	:	FoE.cpp
 *	Version :	1.7
 *	Date	:	2023/06/16
 *	Author	:	User
 *
 *	Hardware dependency function - C source file
 *
 *	Provide necessary function for EcmUsrDriver
 *
 * @copyright (C) 2020 NEXTW TECHNOLOGY CO., LTD.. All rights reserved.
 *
 ******************************************************************************/
#include <windows.h>
#include <iostream>
#include <mutex>
#include <platform.h>
#include <EcmUsrDriver.h>
#include "PdoDefine.h"

uint8_t g_u8RxPDO[1400];
uint8_t g_u8TxPDO[1400];
int FoEDownload(uint8_t slave, char *filename, uint32_t password, uint32_t filesize, uint16_t blocksize, HANDLE fd,	uint32_t timeout)
{
	int i, j = 0;
	uint8_t buf[256];
	int bsize = blocksize & ~3;
	DWORD fsize;
	if (fd < 0)
		return -1;
	SetFilePointer(fd, 0, NULL, 0 /*FILE_BEGIN*/);
	while (filesize > (uint32_t)j) {
		if ((uint32_t)(j + bsize) > filesize) {
			bsize = filesize - j;
		}
		i = ReadFile(fd, buf, bsize, &fsize, NULL);
		if (i == FALSE)
			break;
		UserDelay(1000);
		i = ECM_EcatFoEDataWrite(j, fsize, buf); // fsize must <= 256, SPI dataSize must >= fsize (File to XF Chip via SPI/USB)
		if (i < 0)
			return i;
		j += (fsize + 3)&~3; // j must be a multiple of 4
		printf("%d / %d\r", j, filesize);
	}
	i = ECM_EcatFoEDownload(slave, filename, password, filesize, 0, 10000000); // File from XF IC to slave via FoE
	return i;
}
int main()
{
#pragma region Init
	std::mutex _lock;
	int bLinkPin;
	unsigned short u16SpiSize = 992;
	uint8_t u8State = 0, u8Version = 0, u8TmpEcState = 0, u8CrcErrCnt = 0, u8LastCrcErrCnt = 0, u8FifoCnt = 0;
	uint8_t u8TxFIFOCnt = 64, u8RxFIFOCnt = 64;
	uint16_t u16InfoBuf, u16RxPDOSize, u16TxPDOSize;
	TOTAL_RXPDO_ST_DEF_T	*pRxPDOData;
	TOTAL_TXPDO_ST_DEF_T	*pTxPDOData;
	int i = 0, nRet = 0;
	int nSlvCntOnline;
	memset(g_u8RxPDO, 0, sizeof(g_u8RxPDO));
	memset(g_u8TxPDO, 0, sizeof(g_u8TxPDO));
	pTxPDOData = (TOTAL_TXPDO_ST_DEF_T *)g_u8TxPDO;
	pRxPDOData = (TOTAL_RXPDO_ST_DEF_T *)g_u8RxPDO;
	// 第一個參數為 0 for USB
	// 第一個參數為 1 for PCIe
	XF_SetDevType(0);
	// 初始化PCIe晶片
	//
	nRet = UserSys_Init(24000000);
	if (nRet < 0) {
		printf("UserSys_Init return %d\r\n", nRet);
		return -1;
	}
	u8Version = ECM_InitLibrary(&u16SpiSize, ECM_CRC_TYPE_NONE);
	printf("u8Version 0x%X\r\n", u8Version);
	if (u8Version == 0)
		return 0;
	nRet = ECM_GetNetLinkPin(&bLinkPin);
	printf("ECM_GetNetLinkPin linkpin(%d) ret(%d)\r\n", bLinkPin, nRet); //linkpin=0 代表網路接口沒有連結

	nRet = ECM_EcatInit(0x0, 1000000); // 第一個參數0x300代表DC Sync, 0代表Free Run (依從站ESI說明設置), 第二個參數為週期時間(單位ns)
	printf("ECM_EcatInit %d\r\n", nRet); //return 1 代表 Init成功
#pragma endregion
#pragma region get slave info and set fifo
	nSlvCntOnline = ECM_EcatSlvCntGet();
	printf("Slave %d\n", nSlvCntOnline);
	for (i = 0; i < nSlvCntOnline; i++) {
		nRet = ECM_SlaveInfoGet(i, ECM_SLV_INFO_ALstatuscode, (uint8_t *)&u16InfoBuf);
		if (nRet > 0) {
			if (u16InfoBuf != 0) {
				printf("Slave %d AL StatusCode(%d)...\r\n", i, u16InfoBuf);
			}
		}
		else {
			printf("Slave %d Get AL StatusCode failed...\r\n", i);
		}
	}
	ECM_SetTxFIFOCnt(u8TxFIFOCnt); //Slave -> Master
	ECM_SetRxFIFOCnt(u8RxFIFOCnt); //Master -> Slave
	ECM_InitFIFO();

	// RxPDOSize MUST be the same as RxPDO struct size
	u16RxPDOSize = ECM_FifoRxPdoSizeGet();
	printf("RxPDOSize %d Byte(s), RxPDO struct size %zd Byte(s)\r\n", u16RxPDOSize, sizeof(Total_rxpdo_st_def_t));
	// TxPDOSize MUST be the same as TxPDO struct size
	u16TxPDOSize = ECM_FifoTxPdoSizeGet();
	printf("TxPDOSize %dByte(s), TxPDO struct size %zd Byte(s)\r\n", u16TxPDOSize, sizeof(Total_txpdo_st_def_t));

	nRet = ECM_EcatStateUpdate();
	if (nRet <= 0) {
		printf("ECM_EcatStateUpdate fail(%d)\r\n", nRet);
	}
	else {
		printf("ECM_EcatStateUpdate done.\r\n");
	}
	for (i = 0; i < nSlvCntOnline; i++) {
		nRet = ECM_EcatStateGet(i, &u8TmpEcState);
		if (nRet) {
			printf("(%d) EcState(0x%X)\r\n", i, u8TmpEcState);// u8TmpEcState: 1 Init, 2 preOP, 4 SafeOP, 8 OP
		}
	}
	printf("Any key to continue.");
	getchar();
#pragma endregion
#pragma region FoE for slave 0
	uint32_t fsize;
	ECM_EcatStateSet(0xFF, EC_STATE_INIT);
	Sleep(1000);
	ECM_EcatStateSet(0, EC_STATE_BOOT); // slave 0 to BOOT mode
	Sleep(1000);
	char fileName[] = "ECATFW__";
	HANDLE fd = CreateFileA(
		fileName,
		GENERIC_READ,
		0,
		0,
		OPEN_ALWAYS,
		FILE_ATTRIBUTE_NORMAL,
		NULL
	);
	
	fsize = GetFileSize(fd, NULL); // File size must < 256K 
	if (fsize == 0)
	{
		printf("fileSize = %d, File NOT Found. \r\nAny key to finish.", fsize);
		getchar();
		return 0;
	}
	else
	{
		printf("fileSize = %d\r\nAny key to Start the FoE Download.", fsize);
		getchar();
	}
	printf("\r\n=== Start the FoE Download.===\r\n");
	FoEDownload(0, fileName, 0,	fsize, 256, fd, 1000000); // blockSize must <= 256. SPI Data Size must > blockSize if using SPI interface
	Sleep(1000);
	CloseHandle(fd);
	printf("============================\r\nAny key to finish.\r\n");
	getchar();

#pragma endregion

#pragma region control state to Init and clsoe the system
	ECM_EcatStateSet(0xFF, EC_STATE_INIT);
	Sleep(100);
	ECM_CloseLibrary();
	UserSys_Close();
#pragma endregion
}

