﻿/******************************************************************************
 *	File	:	GetObjectInfo.cpp
 *	Version :	1.7
 *	Date	:	2023/06/06
 *	Author	:	User
 *
 *	Hardware dependency function - C source file
 *
 *	Provide necessary function for EcmUsrDriver
 *
 * @copyright (C) 2020 NEXTW TECHNOLOGY CO., LTD.. All rights reserved.
 *
 ******************************************************************************/
#include <windows.h>
#include <iostream>
#include <mutex>
#include <platform.h>
#include <EcmUsrDriver.h>
#include "PdoDefine.h"

#define OTYPE_VAR               0x0007
#define OTYPE_ARRAY             0x0008
#define OTYPE_RECORD            0x0009

#define ATYPE_Rpre              0x01
#define ATYPE_Rsafe             0x02
#define ATYPE_Rop               0x04
#define ATYPE_Wpre              0x08
#define ATYPE_Wsafe             0x10
#define ATYPE_Wop               0x20

uint8_t g_u8RxPDO[1400];
uint8_t g_u8TxPDO[1400];
int main()
{
#pragma region Init
	std::mutex _lock;
	int bLinkPin;
	unsigned short u16SpiSize = 992;
	uint8_t u8State = 0, u8Version = 0, u8TmpEcState = 0, u8CrcErrCnt = 0, u8LastCrcErrCnt = 0, u8FifoCnt = 0;
	uint8_t u8TxFIFOCnt = 64, u8RxFIFOCnt = 64;
	uint16_t u16InfoBuf, u16RxPDOSize, u16TxPDOSize;
	TOTAL_RXPDO_ST_DEF_T	*pRxPDOData;
	TOTAL_TXPDO_ST_DEF_T	*pTxPDOData;
	int i = 0, nRet = 0;
	int nSlvCntOnline;
	memset(g_u8RxPDO, 0, sizeof(g_u8RxPDO));
	memset(g_u8TxPDO, 0, sizeof(g_u8TxPDO));
	pTxPDOData = (TOTAL_TXPDO_ST_DEF_T *)g_u8TxPDO;
	pRxPDOData = (TOTAL_RXPDO_ST_DEF_T *)g_u8RxPDO;
	// 第一個參數為 0 for USB
	// 第一個參數為 1 for PCIe
	XF_SetDevType(0);
	// 初始化PCIe晶片
	//
	nRet = UserSys_Init(24000000);
	if (nRet < 0) {
		printf("UserSys_Init return %d\r\n", nRet);
		return -1;
	}
	u8Version = ECM_InitLibrary(&u16SpiSize, ECM_CRC_TYPE_NONE);
	printf("u8Version 0x%X\r\n", u8Version);
	if (u8Version == 0)
		return 0;
	nRet = ECM_GetNetLinkPin(&bLinkPin);
	printf("ECM_GetNetLinkPin linkpin(%d) ret(%d)\r\n", bLinkPin, nRet); //linkpin=0 代表網路接口沒有連結

	nRet = ECM_EcatInit(0x0, 1000000); // 第一個參數0x300代表DC Sync, 0代表Free Run (依從站ESI說明設置), 第二個參數為週期時間(單位ns)
	printf("ECM_EcatInit %d\r\n", nRet); //return 1 代表 Init成功
#pragma endregion
#pragma region get slave info and set fifo
	nSlvCntOnline = ECM_EcatSlvCntGet();
	printf("Slave %d\n", nSlvCntOnline);
	for (i = 0; i < nSlvCntOnline; i++) {
		nRet = ECM_SlaveInfoGet(i, ECM_SLV_INFO_ALstatuscode, (uint8_t *)&u16InfoBuf);
		if (nRet > 0) {
			if (u16InfoBuf != 0) {
				printf("Slave %d AL StatusCode(%d)...\r\n", i, u16InfoBuf);
			}
		}
		else {
			printf("Slave %d Get AL StatusCode failed...\r\n", i);
		}
	}
	ECM_SetTxFIFOCnt(u8TxFIFOCnt); //Slave -> Master
	ECM_SetRxFIFOCnt(u8RxFIFOCnt); //Master -> Slave
	ECM_InitFIFO();

	// RxPDOSize MUST be the same as RxPDO struct size
	u16RxPDOSize = ECM_FifoRxPdoSizeGet();
	printf("RxPDOSize %d Byte(s), RxPDO struct size %zd Byte(s)\r\n", u16RxPDOSize, sizeof(Total_rxpdo_st_def_t));
	// TxPDOSize MUST be the same as TxPDO struct size
	u16TxPDOSize = ECM_FifoTxPdoSizeGet();
	printf("TxPDOSize %d Byte(s), TxPDO struct size %zd Byte(s)\r\n", u16TxPDOSize, sizeof(Total_txpdo_st_def_t));

	nRet = ECM_EcatStateUpdate();
	if (nRet <= 0) {
		printf("ECM_EcatStateUpdate fail(%d)\r\n", nRet);
	}
	else {
		printf("ECM_EcatStateUpdate done.\r\n");
	}
	for (i = 0; i < nSlvCntOnline; i++) {
		nRet = ECM_EcatStateGet(i, &u8TmpEcState);
		if (nRet) {
			printf("(%d) EcState(0x%X)\r\n", i, u8TmpEcState);// u8TmpEcState: 1 Init, 2 preOP, 4 SafeOP, 8 OP
		}
	}
	printf("Any key to continue.");
	getchar();
#pragma endregion
#pragma region Get Slave Reg Information
	uint16_t u16d = 0x1;
	uint16_t configadr = 0;
	ECM_SlaveInfoGet(0, ECM_SLV_INFO_configadr, (uint8_t *)&configadr);

	printf("Configured Address Physical Read AL status\n");
	u16d = 0;
	ECM_EcatDatagramReq(ECM_RAW_API_FPRD, configadr, 0x300, 0, 2, 1000000, (uint8_t *)&u16d);
	ECM_EcatDatagramGet((uint8_t *)&u16d);
	printf("Port 0 RX Error Cnt = 0x%02x\n", u16d);

	ECM_EcatDatagramReq(ECM_RAW_API_FPRD, configadr, 0x302, 0, 2, 1000000, (uint8_t *)&u16d);
	ECM_EcatDatagramGet((uint8_t *)&u16d);
	printf("Port 1 RX Error Cnt = 0x%02x\n", u16d);

	ECM_EcatDatagramReq(ECM_RAW_API_FPRD, configadr, 0x304, 0, 2, 1000000, (uint8_t *)&u16d);
	ECM_EcatDatagramGet((uint8_t *)&u16d);
	printf("Port 2 RX Error Cnt = 0x%02x\n", u16d);

	ECM_EcatDatagramReq(ECM_RAW_API_FPRD, configadr, 0x306, 0, 2, 1000000, (uint8_t *)&u16d);
	ECM_EcatDatagramGet((uint8_t *)&u16d);
	printf("Port 3 RX Error Cnt = 0x%02x\n", u16d);
	u16d = 0;
	ECM_EcatDatagramReq(ECM_RAW_API_FPRD, configadr, 0x308, 0, 1, 1000000, (uint8_t *)&u16d);
	ECM_EcatDatagramGet((uint8_t *)&u16d);
	printf("Port 0 Forwarded RX Error Cnt = 0x%02x\n", u16d);

	ECM_EcatDatagramReq(ECM_RAW_API_FPRD, configadr, 0x309, 0, 1, 1000000, (uint8_t *)&u16d);
	ECM_EcatDatagramGet((uint8_t *)&u16d);
	printf("Port 1 Forwarded RX Error Cnt = 0x%02x\n", u16d);

	ECM_EcatDatagramReq(ECM_RAW_API_FPRD, configadr, 0x30A, 0, 1, 1000000, (uint8_t *)&u16d);
	ECM_EcatDatagramGet((uint8_t *)&u16d);
	printf("Port 2 Forwarded RX Error Cnt = 0x%02x\n", u16d);

	ECM_EcatDatagramReq(ECM_RAW_API_FPRD, configadr, 0x30B, 0, 1, 1000000, (uint8_t *)&u16d);
	ECM_EcatDatagramGet((uint8_t *)&u16d);
	printf("Port 3 Forwarded RX Error Cnt = 0x%02x\n", u16d);

	ECM_EcatDatagramReq(ECM_RAW_API_FPRD, configadr, 0x30C, 0, 1, 1000000, (uint8_t *)&u16d);
	ECM_EcatDatagramGet((uint8_t *)&u16d);
	printf("ECAT Processing Unit Error Cnt = 0x%02x\n", u16d);

	ECM_EcatDatagramReq(ECM_RAW_API_FPRD, configadr, 0x310, 0, 1, 1000000, (uint8_t *)&u16d);
	ECM_EcatDatagramGet((uint8_t *)&u16d);
	printf("Port 0 Lost Link Cnt Cnt = 0x%02x\n", u16d);

	ECM_EcatDatagramReq(ECM_RAW_API_FPRD, configadr, 0x311, 0, 1, 1000000, (uint8_t *)&u16d);
	ECM_EcatDatagramGet((uint8_t *)&u16d);
	printf("Port 1 Lost Link Cnt Cnt = 0x%02x\n", u16d);

	ECM_EcatDatagramReq(ECM_RAW_API_FPRD, configadr, 0x312, 0, 1, 1000000, (uint8_t *)&u16d);
	ECM_EcatDatagramGet((uint8_t *)&u16d);
	printf("Port 2 Lost Link Cnt Cnt = 0x%02x\n", u16d);

	ECM_EcatDatagramReq(ECM_RAW_API_FPRD, configadr, 0x313, 0, 1, 1000000, (uint8_t *)&u16d);
	ECM_EcatDatagramGet((uint8_t *)&u16d);
	printf("Port 3 Lost Link Cnt Cnt = 0x%02x\n", u16d);

	ECM_EcatDatagramReq(ECM_RAW_API_FPRD, configadr, 0x110, 0, 2, 1000000, (uint8_t *)&u16d);
	ECM_EcatDatagramGet((uint8_t *)&u16d);
	printf("DL Status = 0x%02x\n", u16d);

	printf("Any key to Broadcast Write AL request to Init.\n");
	getchar();
	u16d = 1;
	ECM_EcatDatagramReq(ECM_RAW_API_BWR, 0, 0x120, 0, 2, 1000000, (uint8_t *)&u16d);

	printf("Any key to Configured Address Physical Write AL request to PerOP\n");
	getchar();	   	 
	u16d = 2;
	ECM_SlaveInfoGet(0, ECM_SLV_INFO_configadr, (uint8_t *)&configadr);
	ECM_EcatDatagramReq(ECM_RAW_API_FPWR, configadr, 0x120, 0, 2, 1000000, (uint8_t *)&u16d);

	printf("Any key to Configured Address Physical Read AL status\n");
	getchar();	
	u16d = 0;
	ECM_EcatDatagramReq(ECM_RAW_API_FPRD, configadr, 0x130, 0, 2, 1000000, (uint8_t *)&u16d);
	ECM_EcatDatagramGet((uint8_t *)&u16d);
	printf("AL status = 0x%02x\n", u16d);
	printf("============================\r\n Any key to finish.");
	getchar();
#pragma endregion

#pragma region control state to Init and clsoe the system
	ECM_EcatStateSet(0xFF, EC_STATE_INIT);
	Sleep(100);
	ECM_CloseLibrary();
	UserSys_Close();
#pragma endregion
}


