﻿/******************************************************************************
 *	File	:	IO.cpp
 *	Version :	1.7
 *	Date	:	2022/07/27
 *	Author	:	User
 *
 *	Hardware dependency function - C source file
 *
 *	Provide necessary function for EcmUsrDriver
 *
 * @copyright (C) 2020 NEXTW TECHNOLOGY CO., LTD.. All rights reserved.
 *
 ******************************************************************************/
#include <windows.h>
#include <iostream>
#include <mutex>
#include <platform.h>
#include <EcmUsrDriver.h>
#include "PdoDefine.h"
uint8_t g_u8RxPDO[1400];
uint8_t g_u8TxPDO[1400];

int main()
{
#pragma region Init
	std::mutex _lock;
	int bLinkPin;
	unsigned short u16SpiSize = 992;
	uint8_t u8State = 0, u8Version = 0, u8TmpEcState = 0, u8CrcErrCnt = 0, u8LastCrcErrCnt = 0, u8FifoCnt = 0;
	uint8_t u8TxFIFOCnt = 64, u8RxFIFOCnt = 64;
	uint16_t u16InfoBuf, u16RxPDOSize, u16TxPDOSize;
	TOTAL_RXPDO_ST_DEF_T	*pRxPDOData;
	TOTAL_TXPDO_ST_DEF_T	*pTxPDOData;
	int i = 0, nRet = 0;
	int nSlvCntOnline;
	memset(g_u8RxPDO, 0, sizeof(g_u8RxPDO));
	memset(g_u8TxPDO, 0, sizeof(g_u8TxPDO));
	pTxPDOData = (TOTAL_TXPDO_ST_DEF_T *)g_u8TxPDO;
	pRxPDOData = (TOTAL_RXPDO_ST_DEF_T *)g_u8RxPDO;
	// 第一個參數為 0 for USB
	// 第一個參數為 1 for PCIe
	XF_SetDevType(0);
	// 初始化PCIe晶片
	//
	nRet = UserSys_Init(24000000);
	if (nRet < 0) {
		printf("UserSys_Init return %d\r\n", nRet);
		return -1;
	}
	u8Version = ECM_InitLibrary(&u16SpiSize, ECM_CRC_TYPE_NONE);
	printf("u8Version 0x%X\r\n", u8Version);
	if (u8Version == 0)
		return 0;
	nRet = ECM_GetNetLinkPin(&bLinkPin);
	printf("ECM_GetNetLinkPin linkpin(%d) ret(%d)\r\n", bLinkPin, nRet); //linkpin=0 代表網路接口沒有連結

	nRet = ECM_EcatInit(0x0, 1000000); // 第一個參數0x300代表DC Sync, 0代表Free Run (依從站ESI說明設置), 第二個參數為週期時間(單位ns)
	printf("ECM_EcatInit %d\r\n", nRet); //return 1 代表 Init成功
#pragma endregion
#pragma region get slave info and set fifo and PDO maps
	nSlvCntOnline = ECM_EcatSlvCntGet();
	printf("Slave %d\n", nSlvCntOnline);
	for (i = 0; i < nSlvCntOnline; i++) {
		nRet = ECM_SlaveInfoGet(i, ECM_SLV_INFO_ALstatuscode, (uint8_t *)&u16InfoBuf);
		if (nRet > 0) {
			if (u16InfoBuf != 0) {
				printf("Slave %d AL StatusCode(%d)...\r\n", i, u16InfoBuf);
			}
		}
		else {
			printf("Slave %d Get AL StatusCode failed...\r\n", i);
		}
	}
	ECM_SetTxFIFOCnt(u8TxFIFOCnt); //Slave -> Master
	ECM_SetRxFIFOCnt(u8RxFIFOCnt); //Master -> Slave
	ECM_InitFIFO();

	// RxPDOSize MUST be the same as RxPDO struct size
	u16RxPDOSize = ECM_FifoRxPdoSizeGet();
	printf("RxPDOSize %d Byte(s), RxPDO struct size %zd Byte(s)\r\n", u16RxPDOSize, sizeof(Total_rxpdo_st_def_t));
	// TxPDOSize MUST be the same as TxPDO struct size
	u16TxPDOSize = ECM_FifoTxPdoSizeGet();
	printf("TxPDOSize %dByte(s), TxPDO struct size %zd Byte(s)\r\n", u16TxPDOSize, sizeof(Total_txpdo_st_def_t));

	nRet = ECM_EcatStateUpdate();
	if (nRet <= 0) {
		printf("ECM_EcatStateUpdate fail(%d)\r\n", nRet);
	}
	else {
		printf("ECM_EcatStateUpdate done.\r\n");
	}
	for (i = 0; i < nSlvCntOnline; i++) {
		nRet = ECM_EcatStateGet(i, &u8TmpEcState);
		if (nRet) {
			printf("(%d) EcState(0x%X)\r\n", i, u8TmpEcState);// u8TmpEcState: 1 Init, 2 preOP, 4 SafeOP, 8 OP
		}
	}
	printf("Any key to continue.");
	getchar();
#pragma endregion
#pragma region control state from perOP to OP
	ECM_EcatStateSet(0xFF, EC_STATE_SAFE_OP);
	Sleep(1000);
	nRet = ECM_EcatStateGet(0xFF, &u8TmpEcState);
	if (nRet) {
		printf("EcState(0x%X)\r\n", u8TmpEcState);
	}
	ECM_EcatStateSet(0xFF, EC_STATE_OPERATIONAL);
	for (i = 0; i < 10; i++) {
		Sleep(1000);
		nRet = ECM_EcatStateGet(0xFF, &u8TmpEcState);
		if (nRet > 0) {
			printf("EcState(0x%X)\r\n", u8TmpEcState);
		}
		if (u8TmpEcState == 8) {// u8TmpEcState: 1 Init, 2 preOP, 4 SafeOP, 8 OP
			break;
		}
	}
#pragma endregion
#pragma region Cyclic PDO change using FIFO
	if (u8TmpEcState == 8) {  // check EcState is OP 
		u8CrcErrCnt = 0;
		pRxPDOData->IO_GROUP_0[0].u16Output = 0;
		uint16_t LastInput = 0;
		for (i = 0; i < 100000; )
		{
			if (i % 100 == 0)
			{
				pRxPDOData->IO_GROUP_0[0].u16Output++; // Set Target Velocity 
			}
			if (u8FifoCnt < u8RxFIFOCnt - 3)
			{
				_lock.lock();
				nRet = ECM_EcatPdoFifoDataExchangeAdv(ECM_FIFO_RW, 1, g_u8RxPDO, g_u8TxPDO, u16RxPDOSize, &u8FifoCnt, &u8CrcErrCnt, NULL, NULL);
				_lock.unlock();
				if (pTxPDOData->IO_GROUP_0[0].u16Input != LastInput)
				{
					LastInput = pTxPDOData->IO_GROUP_0[0].u16Input;
					printf("time %d:\tFifoCnt %d, CrcErrCnt %d, Ret %d, input 0x%X      \r\n", i, u8FifoCnt, u8CrcErrCnt, nRet, pTxPDOData->IO_GROUP_0[0].u16Input);
				}
				if (nRet <= 4) {
					printf("(%d)\tFifoCnt %d Crc %d Ret %d\r\n", i, u8FifoCnt, u8CrcErrCnt, nRet);
				}
				else {
					i++;
				}
			}
			else
			{
				_lock.lock();
				nRet = ECM_EcatPdoFifoDataExchangeAdv(ECM_FIFO_RD, 1, g_u8RxPDO, g_u8TxPDO, u16RxPDOSize, &u8FifoCnt, &u8CrcErrCnt, NULL, NULL);
				_lock.unlock();
				printf("time %d:\tFifoCnt %d, CrcErrCnt %d, Ret %d, input 0x%X      \r", i, u8FifoCnt, u8CrcErrCnt, nRet, pTxPDOData->IO_GROUP_0[0].u16Input);
				Sleep(10);
			}
		}
		printf("\n Finish \tFifoCnt %d, CrcErrCnt %d, Ret %d\n", u8FifoCnt, u8CrcErrCnt, nRet);
		printf("Any key to close.\n");
		getchar();
	}
#pragma endregion
#pragma region control state from OP to Init and clsoe the system
	ECM_EcatStateSet(0xFF, EC_STATE_SAFE_OP);
	Sleep(100);
	ECM_EcatStateSet(0xFF, EC_STATE_PRE_OP);
	Sleep(100);
	ECM_EcatStateSet(0xFF, EC_STATE_INIT);
	Sleep(100);
	ECM_CloseLibrary();
	UserSys_Close();
#pragma endregion
}

// 執行程式: Ctrl + F5 或 [偵錯] > [啟動但不偵錯] 功能表
// 偵錯程式: F5 或 [偵錯] > [啟動偵錯] 功能表

// 開始使用的提示: 
//   1. 使用 [方案總管] 視窗，新增/管理檔案
//   2. 使用 [Team Explorer] 視窗，連線到原始檔控制
//   3. 使用 [輸出] 視窗，參閱組建輸出與其他訊息
//   4. 使用 [錯誤清單] 視窗，檢視錯誤
//   5. 前往 [專案] > [新增項目]，建立新的程式碼檔案，或是前往 [專案] > [新增現有項目]，將現有程式碼檔案新增至專案
//   6. 之後要再次開啟此專案時，請前往 [檔案] > [開啟] > [專案]，然後選取 .sln 檔案
